% Partially open box arrays with L layers POBA-L
%
% Written by Chun-Lin Liu
% E-mail : cl.liu@caltech.edu
%
% Our Papers:
% [1] C.-L. Liu and P. P. Vaidyanathan, “Two-Dimensional Sparse Arrays with Hole-Free Coarray and Reduced Mutual Coupling,” in Proc. of 50th IEEE Asilomar Conference on Signals, Systems, and Computers, Pacific Grove, CA, Nov. 2016.
% [2] C.-L. Liu and P. P. Vaidyanathan, “Hourglass Arrays, and Other Novel 2D Sparse Arrays with Reduced Mutual Coupling,” submitted to IEEE Transactions on Signal Processing.
%
% Last revised on August 21, 2016


function S = POBA_L( varargin )
%POBA_L Summary of this function goes here
%   Returns partially open box arrays with L layers
%   Possible input combinations:
%   1) S = POBA_L( Nx, Ny, 'OBA' )     { or POBA_L( Nx, Ny, 'open' ) }
%       Open box arrays (OBA)
%   2) S = POBA_L( Nx, Ny, 'HOBA' )    { or POBA_L( Nx, Ny, 'half' ) }
%       Half open box arrays (HOBA)
%   3) S = POBA_L( Nx, Ny, 'HOBA-2' )  { or POBA_L( Nx, Ny, 'half-two') }
%       Half open box arrays with two layers (HOBA-2)
%   4) S = POBA_L( Nx, Ny, 'hourglass' )
%       Hourglass arrays
%   5) S = POBA_L( Nx, Ny, g_1 )
%       Partially open box array with g_1
%   6) S = POBA_L( Nx, Ny, g_1, h_1 )
%       Partially open box array with g_1, h_1{1}, ..., h_1{L}
%
%   Output: An integer matrix S. The first column S(:,1) contains all the x
%   coordinates while the second column S(:,2) has all the y coordinates.

switch (nargin)
    case 3
        Nx = double(varargin{1});
        Ny = double(varargin{2});
        if ( strcmpi( varargin{3}, 'OBA') || strcmpi( varargin{3}, 'open') )
            % ========== Open box arrays ==========
            g_1 = (1:Nx-2).';
            S = POBA_L( Nx, Ny, g_1 );
        elseif (strcmpi( varargin{3}, 'HOBA') || strcmpi( varargin{3}, 'half'))
            % ========== Half open box arrays ==========
            g_1 = 1 + 2 * (0:floor((Nx-3)/2));
            S = POBA_L( Nx, Ny, g_1 );
        elseif (strcmpi( varargin{3}, 'HOBA-2') || strcmpi( varargin{3}, 'half-two') )
            % ========== Half Open Box Arrays with two layers ==========
            g_1 = 1 + 2 * (0:floor((Nx-3)/2));
            h_1 = cell(2, 1);
            h_1{2} = 2 * (1 : floor((Ny-3)/2));
            h_1{1} = setdiff( 1 : Ny-2, h_1{2} );
            S = POBA_L( Nx, Ny, g_1, h_1 );
        elseif (strcmpi( varargin{3}, 'hourglass') )
            % ========== Hourglass Arrays ==========
            g_1 = 1 + 2 * (0:floor((Nx-3)/2));
            if (mod(Ny, 2) == 1)
                L = floor((Ny+1)/4);
                h_1 = cell(L, 1);
                for ll = 2 : L
                    h_1{ll} = [2*ll-1; Ny-2*ll];
                end
            else
                L = floor(Ny/8+1);
                h_1 = cell(L, 1);
                for ll = 2 : L
                    h_1{ll} = [2*ll-1;
                        2*floor(Ny/4)-2*ll+3;
                        2*ceil(Ny/4)+2*ll-4;
                        Ny-2*ll];
                end
            end
            if (Nx < 2*L)
                error('Hourglass arrays are not well-defined when Nx < 2L')
            else
                u = (1 : floor((Ny-1)/4)).';
                h_1{1} = unique([2*u; Ny-1-2*u; 1; Ny-2]);
                S = POBA_L( Nx, Ny, g_1, h_1 );
            end
        else
            g_1 = varargin{3};
            if ( ~isempty(g_1) )
                % Case I: g_1 is empty, then g_2 = {1, ..., N_y-2}.
                % Case II: g_1 is not empty. g_1 has to be a vector
                if (~isvector(g_1) )
                    error('g_1 has to be a vector')
                end
                % If g_1 is a row vector, then transpose it first
                if ( isrow(g_1) )
                    g_1 = g_1.';
                end
                g_1 = unique(g_1);
                if (max(g_1) > Nx-2 || min(g_1) < 1)
                    error('g_1 does not satisfy 1 <= every element in g_1 <= Nx-2')
                end
            end
            g_2 = Nx - 1 - setdiff((1 : Nx-2).', g_1);
            S = [0, 0;
                Nx-1, 0;
                0, Ny-1;
                Nx-1, Ny-1;
                g_1, zeros(size(g_1));
                g_2, (Ny-1)*ones(size(g_2));
                zeros(Ny-2, 1), (1:Ny-2).';
                (Nx-1)*ones(Ny-2, 1), (1:Ny-2).'];
        end
    case 4
        Nx = double(varargin{1});
        Ny = double(varargin{2});
        
        g_1 = varargin{3};
        if ( ~isempty(g_1) )
            % Case I: g_1 is empty, then g_2 = {1, ..., N_y-2}.
            % Case II: g_1 is not empty. g_1 has to be a vector
            if (~isvector(g_1) )
                error('g_1 has to be a vector')
            end
            % If g_1 is a row vector, then transpose it first
            if ( isrow(g_1) )
                g_1 = g_1.';
            end
            g_1 = unique(g_1);
            if (max(g_1) > Nx-2 || min(g_1) < 1)
                error('g_1 does not satisfy 1 <= every element in g_1 <= Nx-2')
            end
        end
        g_2 = Nx - 1 - setdiff((1 : Nx-2).', g_1);
        S = [0, 0;
            Nx-1, 0;
            0, Ny-1;
            Nx-1, Ny-1;
            g_1, zeros(size(g_1));
            g_2, (Ny-1)*ones(size(g_2))];
        
        h_1 = varargin{4};
        L = length(h_1);
        % { h_1{ll} } needs to be a partition of {1, ..., Ny-2}
        H = []; % union of h_1{ll}
        for ll = 1 : L
            if ( ~isempty(h_1{ll}) )
                % h_1{ll} has to be a vector
                if (~isvector(h_1{ll}) )
                    error(['h_{1,', num2str(ll), '} has to be a vector'])
                end
                % If h_1{ll} is a row vector, then transpose it first
                if ( isrow(h_1{ll}) )
                    h_1{ll} = h_1{ll}.';
                end
                % H and h{ll} have to be disjoint
                if ( ~isempty( intersect(H, h_1{ll}) ) )
                    error('h_1{ll} are not disjoint.' )
                end
            end
            H = [H; h_1{ll}];
        end
        % H should be {1, ..., Ny-2}
        if (~isequal(unique(H), (1:Ny-2).'))
            error('h_1{ll} is not a partition of {1, ..., N_y-2}')
        end
        for ll = 1 : L
            S = [S;
                (ll-1)*ones(size(h_1{ll})), h_1{ll}; % H_{1,ll}
                (Nx-ll)*ones(size(h_1{ll})), Ny-1-h_1{ll}]; % H_{2, ll}
        end
        
    otherwise
        error('Invalid input parameters');
end

% Remove duplicated rows
S = unique(S, 'rows');

end

