function [ D, w, n1, n2 ] = weight_function_2D( S, coarray_set )
%WEIGHT_FUNCTION_2D: Evaluate the weight function of a 2D sparse array
%
% Written by Chun-Lin Liu
% E-mail : cl.liu@caltech.edu
%
% Last revised on October 9, 2015
%
%   Input:
%       1) S: The discrete integer set
%       2) coarray_set: specify the output grid D. It has three options
%           'D': w is defined over D
%           'V': defined over V, including the holes
%
%   Output:
%       1) D: Output grid
%       2) w: weight function on D (2D)
%       3) n1, n2: A cell array for sensor pairs( n1, n2) such that m = n1 - n2. 
%
%
%   Example:
%       S = [0, 0; 1, 0; 0, 1];
%       1) [ D, w ] = weight_function_2D( S, 'V' ) gives
%           D = [-1, -1;
%                 0, -1;
%                 1, -1;
%                -1,  0;
%                 0,  0;
%                 1,  0;
%                -1,  1;
%                 0,  1;
%                 1,  1];
%           w = [0; 1; 1; 1; 3; 1; 1; 1; 0]; 
%           i.e., w(-1, -1) = 0, w(0, -1) = 1, ...
%
%       2) [ D, w ] = weight_function_2D( S, 'D' ) gives
%           D = [ 0, -1;
%                 1, -1;
%                -1,  0;
%                 0,  0;
%                 1,  0;
%                -1,  1;
%                 0,  1];
%           w = [1; 1; 1; 3; 1; 1; 1];



if (strcmpi(coarray_set, 'V'))
    MAX_x = max(S(:, 1));
    MIN_x = min(S(:, 1));
    MAX_y = max(S(:, 2));
    MIN_y = min(S(:, 2));
    [mx_mat, my_mat] = ndgrid(-(MAX_x - MIN_x):(MAX_x - MIN_x), -(MAX_y - MIN_y):(MAX_y - MIN_y));
    mx_vec = mx_mat(:); my_vec = my_mat(:);
    w = zeros(size(mx_vec));
    n1 = cell(size(mx_vec));
    n2 = cell(size(mx_vec));
    for ii = 1 : size(S, 1)
        for jj = 1 : size(S, 1)
            index = (mx_vec == S(ii, 1) - S(jj, 1) & my_vec == S(ii, 2) - S(jj, 2));
            w( index ) = w( index ) + 1;
            n1{ index } = [n1{ index }; S(ii, :)];
            n2{ index } = [n2{ index }; S(jj, :)];
        end
    end
    D = [mx_vec, my_vec];
elseif (strcmpi(coarray_set, 'D'))
    [V, w_V, n1_V, n2_V] = weight_function_2D( S, 'V' );
    index = (w_V > 0);
    D = V( index, : );
    w = w_V( index);
    n1 = n1_V{ index };
    n2 = n2_V{ index };
else
end







end

